/*
c------------------------------------------------------------------------------
c                Constant Volatility American Call Binomial Model
c------------------------------------------------------------------------------
c     Routine name    - binomc 
c------------------------------------------------------------------------------
c     Purpose         - This subroutine calculates the value of an American 
c                       call with 1 dividend using the binomial numerical
c                       procedure of Cox and Ross, and escrowed dividends. 
c------------------------------------------------------------------------------
c     Arguments: 
c         sp --- initial stock price        sk --- exercise(striking) price
c         r  --- risk-free rate             t  --- time to maturity
c         v  --- initial volatility       tdiv --- time to dividend 
c         div --- dividend amount   reps --- number of discreted time steps
c         cval  --- call price (output)
c------------------------------------------------------------------------------
c     Required routines - None
c------------------------------------------------------------------------------
*/
#ifndef Models_H
#define Models_H
 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#define MAXSIZE 1000
#define  P 0.5

/* function prototypes */

float binomc(sp,sk,r,t,v,tdiv,div,reps)
     float sp,sk,r,t,v,tdiv,div;
     int reps;
{
  float r1,u,d,a1,a2,s0,escrow,xx,dt,tmp1,tmp2,pvdiv;
  float c[MAXSIZE];
  int i,j,n1,n2;
/*  extern  exp();*/

/*c     ---  set formula parameters */
  dt    = t/(float)reps;
  r1    = pow((r + 1.0),dt);
  tmp1  = (r - 0.5*v*v)*dt;
  tmp2  = v*sqrt(dt);
  u     = tmp1 + tmp2;
  d     = tmp1 - tmp2;
  n2    = (int)(tdiv*365.0);
  pvdiv = div/pow(r1,tdiv*reps/t);
  s0    = log(sp - pvdiv);

  /*     ---  calculate binomial call value */
  for ( i = 0; i<=reps; i++) {
    
    /*    ---  calculate reduction in stock price due to dividends */
    n1 = (int)((reps-i)*t*365.0/reps);
      if (n1 >= n2) 
	escrow = 0.0;
      else
	escrow = pvdiv*pow(r1,(reps-i));
    
    /*     ---  evaluate binomial call formula recursively */
    for ( j = 0; j<=( reps - i); j++) {
      xx = j*u + (reps-i-j)*d + s0;
      xx = (xx>=20.0) ? 20.0 : xx;
      a1 = exp(xx) + escrow - sk;
      
      if (i == 0) 
	a2 = 0.0;
      else
	a2 = (P*c[j+2] + (1.0-P) * c[j+1])/r1;
      c[j+1] = (a1>=a2) ? a1 : a2;
    }
  }
  
  return c[1];
}
#endif Models_H
